import tkinter as tk
from tkinter import ttk
from SummarizerModel import SummarizerModel
import threading


class SummarizerGUI:
    def __init__(self, master):
        self.master = master
        self.master.title("Text Summarizer")

        self.model_options = ["google/flan-t5-small", "google/flan-t5-base", "google/flan-t5-large", "google/flan-t5-xxl", "facebook/bart-large-cnn", "sshleifer/distilbart-cnn-12-6", "google/pegasus-xsum"]
        self.selected_model = tk.StringVar()
        self.selected_model.set(self.model_options[0])

        self.model_label = tk.Label(master, text="Select Model:")
        self.model_dropdown = ttk.Combobox(master, values=self.model_options, textvariable=self.selected_model)
        self.model_dropdown.bind("<<ComboboxSelected>>", self.on_model_selected)

        self.text_entry_label = tk.Label(master, text="Enter Text:")
        self.text_entry = tk.Text(master, height=10, width=50)

        self.summary_label = tk.Label(master, text="Summary:")
        self.summary_text = tk.Text(master, height=5, width=50, state=tk.DISABLED)

        self.summarizer_model = None

        self.summarize_button = tk.Button(master, text="Summarize", command=self.initialize_model)

        self.model_label.grid(row=0, column=0, padx=10, pady=10)
        self.model_dropdown.grid(row=0, column=1, padx=10, pady=10)
        self.text_entry_label.grid(row=1, column=0, padx=10, pady=10)
        self.text_entry.grid(row=1, column=1, padx=10, pady=10)
        self.summary_label.grid(row=2, column=0, padx=10, pady=10)
        self.summary_text.grid(row=2, column=1, padx=10, pady=10)
        self.summarize_button.grid(row=3, column=1, pady=10)

        # Add a loop to allow multiple text entries and model changes
        self.master.protocol("WM_DELETE_WINDOW", self.on_close)

    def initialize_model(self):
        self.text_entry.config(state=tk.DISABLED)
        self.summarize_button.config(state=tk.DISABLED)
        threading.Thread(target=self.load_model).start()

    def load_model(self):
        self.master.after(0, self.load_model_async)

    def load_model_async(self):
        selected_model = self.model_dropdown.get()
        if self.summarizer_model is None or self.summarizer_model.model_name != selected_model:
            self.summarizer_model = SummarizerModel(model_name=selected_model)
        self.text_entry.config(state=tk.NORMAL)
        self.summarize_button.config(state=tk.NORMAL)
        self.summarize_text()

    def summarize_text(self):
        input_text = self.text_entry.get("1.0", tk.END)
        selected_model = self.model_dropdown.get()
        tokenized_text = self.summarizer_model.tokenize(input_text)
        summary = self.summarizer_model.summarize(tokenized_text)
        self.summary_text.config(state=tk.NORMAL)
        self.summary_text.delete("1.0", tk.END)
        self.summary_text.insert(tk.END, summary[0])
        self.summary_text.config(state=tk.DISABLED)

    def on_model_selected(self, event):
        # Callback when the model is selected from the dropdown
        self.initialize_model()

    def on_close(self):
        self.master.destroy()


if __name__ == '__main__':
    root = tk.Tk()
    app = SummarizerGUI(root)
    root.mainloop()