from transformers import AutoModelForSeq2SeqLM, PreTrainedTokenizer, AutoTokenizer, BatchEncoding, GenerationConfig, \
    T5PreTrainedModel
from typing import Tuple, List


class SummarizerModel:
    def __init__(self, model_name: str = "google/flan-t5-base"):
        self.tokenizer = AutoTokenizer.from_pretrained(model_name)
        self.model = AutoModelForSeq2SeqLM.from_pretrained(model_name)
        self.model_name = model_name  # Store the model name

    def tokenize(self, text: str, prompt_template: str = "Summarize the following text:\n{} Summary:") -> BatchEncoding:
        tokenized_text = self.tokenizer(prompt_template.format(text),
                                        truncation=True,
                                        padding=False,
                                        return_tensors='pt')
        return tokenized_text

    def summarize(self, tokenized_text: BatchEncoding, num_summaries: int = 1) -> List[str]:
        config = GenerationConfig(
            max_new_tokens=200,
            use_cache=True,
            num_return_sequences=num_summaries
        )
        result = self.model.generate(**tokenized_text, generation_config=config)
        return self.tokenizer.batch_decode(result, skip_special_tokens=True)
